<?php

namespace GopNext\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class OperationalBudgetApprovalController {

    protected $crudService;
  
    public function __construct($connection, $environment, $util, $crudService, $service) {
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->util         = $util;
        $this->crudService	= $crudService;
        $this->service      = $service;
    }
    
    public function approvalBudgetFilter(Request\Filter $request, Response $response) {
        $params = ['CDOPERADOR', 'NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        foreach ($filter as $key => &$item) {
            if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                $item["value"] = $item["value"]["NRPERIODOORCA"];
            }
        }

        $data = $this->crudService->getFilteredDataSet('approvalBudgetGop', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("approvalBudgetGop", $data));
    }
    
    public function updateBudgetStatus(Request\Row $request, Response $response) {
        $dataRow = $request->getRow();
        $rows = $dataRow['SELECTEDITEMS'];
        $activity = $dataRow['ACTIVITYTYPE'];
        $observation = $dataRow['DESCRIPTIONMODAL'];
        
        try{
            $this->service->beginTransaction();
            foreach($rows as $row) {
                if($activity == 'Approve') {
                    if(isset($row['NRAPROVAORCAGOP'])) {
                        $this->updateAprovationStatus($row, $activity, $observation);
                    } else {
                        $params = array(
                           'P_NRORCAMENTOGOP'       => $row['NRORCAMENTOGOP'],
                           'P_CDOPERADOR'           => $this->environment->getCdOperador(),
                           'P_IDAPROVACAOGOP'       => 'APROVADO',
                           'P_DSOBSERVACAO'         => $observation,
                           'P_CDOPERINCLUSAO'       => $this->environment->getCdOperador(),
                           'P_IDTIPOORCAOP'         => $row['IDTIPOORCAOP']
                        );
                        $this->service->insertNewAprovation($params);
                    }
                } else if ($activity == 'Reprove' || $activity == 'Cancel') {
                    $params = array(
                        'P_IDSTATUS'            => 'PENDENTE',
                        'P_DSOBSERVACAO'        => $observation,
                        'P_CDFILIAL'            => $row['CDFILIAL'],
                        'P_NRCONCPREORCAGOP'    => $row['NRCONCPREORCAGOP']
                    );
                    $this->service->updateConclusaoStatus($params);
                    $this->updateAprovationStatus($row, $activity, $observation);
                }
            }
            $this->service->commit();
           
        //   $retorno = '';
        //   $procedure = new StoredProcedure($this->connection, 'CONSISTE_ORCAMENTO_GOP');
    
        //   $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $rows[0]['CDFILIAL'],   Param::PARAM_TYPE_STR));
        //   $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, null,                     Param::PARAM_TYPE_STR));
        //   $procedure->addParam(new Param('P_NRDEMANDA', Param::PARAM_INPUT, null,                     Param::PARAM_TYPE_STR));
        //   $procedure->addParam(new Param('P_RETORNO',   Param::PARAM_OUTPUT,$retorno,                 Param::PARAM_TYPE_STR));
        //   $procedure->addParam(new Param('P_NRORG',     Param::PARAM_INPUT, $this->environment->getNrorg(), Param::PARAM_TYPE_STR));
    
        //   $procedure->execute();
           
        } catch(\Exception $ex) {
           $this->service->rollback();
           $response->setError(new Error("Não foi possível processar os dados enviados" . $ex->getMessage(), 500));
       }
    }
    
    private function updateAprovationStatus($row, $activity, $observation) {
        if($activity == 'Approve') {
            $row['IDAPROVACAOGOP'] = 'APROVADO';
        }else if($activity == 'Cancel'){
            $row['IDAPROVACAOGOP'] = 'PENDENTE';
        }else if($activity == 'Reprove') {
            $row['IDAPROVACAOGOP'] = 'REPROVADO';
        }
        
        $params = array(
            'P_NRAPROVAORCAGOP'     => $row['NRAPROVAORCAGOP'],
            'P_IDAPROVACAOGOP'      => $row['IDAPROVACAOGOP'],
            'P_DSOBSERVACAO'        => $observation,
            'P_NRORGULTATU'         => $this->environment->getNrorg(),
            'P_CDOPERULTATU'        => $this->environment->getCdOperador()
        );
        
        $this->service->updateAprovationStatus($params);
    }
}
