<?php
namespace GopNext\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class CalculationBudgetService {
    
    public function __construct($entityManager, $environment){
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
  
    /**
    * Start transaction on database server
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
  
    /**
    * Commit transaction on database server
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function commit() {
        $this->connection->commit();
    }
  
    /**
    * Roll back transaction in database
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function rollback() {
        $this->connection->rollback();
    }

    /**
    * Make changes to the monthly budget
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function changeBudgetMonth($params) {
        $paramsMonth = [
            "NRORCAMENTOGOP" => $params["NRORCAMENTOGOP"],
            "IDTIPOORCAOP"   => $params["IDTIPOORCAOP"],
            "IDEDITMES"      => $params["IDEDITMES"],
            "NRORG"          => $params["NRORG"]
        ];
                
        $query = 
            "SELECT I.IDTIPOORCAOP, I.NRITEMORCAOP, M.NRITORCAMESGOP, M.DTORCAMENTO,
                    M.IDALTMANUAL, TO_NUMBER(TO_CHAR(M.DTORCAMENTO,'MM')) NRMES
               FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                           AND I.IDTIPOORCAOP = :IDTIPOORCAOP
                                   INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                                                          AND TO_CHAR(M.DTORCAMENTO,'YYYY/MM') >= :IDEDITMES
              WHERE O.NRORCAMENTOGOP = :NRORCAMENTOGOP
                AND O.NRORG = :NRORG
                AND O.IDVERSORCAGOP = 'NEXT'
              ORDER BY I.NRITEMORCAOP";
        try {
            $stmt = $this->connection->prepare($query);
            $stmt->execute($paramsMonth);
            $records = $stmt->fetchAll();
            $amount = 0;
            $nrItemOrcaOp = 0;
            
            foreach($records as $record){
                if ($nrItemOrcaOp !== 0 && $nrItemOrcaOp !== $record["NRITEMORCAOP"]) {
                    $this->updatesTotals($amount, $record["NRITEMORCAOP"], $params["NRORG"]);
                    $amount = 0;
                }
                $nrItemOrcaOp = $record["NRITEMORCAOP"];
                
                $update = "UPDATE ITORCAMESGOP SET VRORCAMENTO = :VRORCAMENTO
                            WHERE NRITORCAMESGOP = :NRITORCAMESGOP
                              AND NRORG = :NRORG";
                
                $paramsUpd = [
                    "NRITORCAMESGOP" => $record["NRITORCAMESGOP"],
                    "NRORG" => $paramsMonth["NRORG"]
                ];
                switch ($record["NRMES"]) {
                    case 1:
                        $paramsUpd["VRORCAMENTO"] = $params["JANEIRO"];
                        break;
                    case 2:
                        $paramsUpd["VRORCAMENTO"] = $params["FEVEREIRO"];
                        break;
                    case 3:
                        $paramsUpd["VRORCAMENTO"] = $params["MARCO"];
                        break;
                    case 4:
                        $paramsUpd["VRORCAMENTO"] = $params["ABRIL"];
                        break;
                    case 5:
                        $paramsUpd["VRORCAMENTO"] = $params["MAIO"];
                        break;
                    case 6:
                        $paramsUpd["VRORCAMENTO"] = $params["JUNHO"];
                        break;
                    case 7:
                        $paramsUpd["VRORCAMENTO"] = $params["JULHO"];
                        break;
                    case 8:
                        $paramsUpd["VRORCAMENTO"] = $params["AGOSTO"];
                        break;
                    case 9:
                        $paramsUpd["VRORCAMENTO"] = $params["SETEMBRO"];
                        break;
                    case 10:
                        $paramsUpd["VRORCAMENTO"] = $params["OUTUBRO"];
                        break;
                    case 11:
                        $paramsUpd["VRORCAMENTO"] = $params["NOVEMBRO"];
                        break;
                    case 12:
                        $paramsUpd["VRORCAMENTO"] = $params["DEZEMBRO"];
                        break;
                }
                $amount = $amount + $paramsUpd["VRORCAMENTO"];
                
                $stmt = $this->connection->prepare($update);
                $stmt->execute($paramsUpd);
            }
            
            $this->updatesTotals($amount, $nrItemOrcaOp, $params["NRORG"]);
        } catch (Exception $ex) {
            throw $ex;
        };
    }

    /**
    * Updates totals regarding changes applied in the month
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function updatesTotals($value, $nrItemOrcaOp, $nrOrg) {
        try {
            
            $update = "UPDATE ITORCAMESGOP SET VRTOTALMES = :VRTOTALMES
                        WHERE NRITEMORCAOP = :NRITEMORCAOP
                          AND NRORG = :NRORG";
                
            $paramsUpd = [
                "VRTOTALMES"   => $value,
                "NRITEMORCAOP" => $nrItemOrcaOp,
                "NRORG"        => $nrOrg
            ];
            
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
            
            $update = "UPDATE ITEMORCAGOP SET VRTOTALMES = :VRTOTALMES
                        WHERE NRITEMORCAOP = :NRITEMORCAOP
                          AND NRORG = :NRORG";
                
            $paramsUpd = [
                "VRTOTALMES"   => $value,
                "NRITEMORCAOP" => $nrItemOrcaOp,
                "NRORG"        => $nrOrg
            ];
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
        } catch (Exception $ex) {
            throw $ex;
        }
    }

    /**
    * Triggers budget calculations such as calculation by percentage, by previous periods,
    * among others used in the CALCULA_ORCAMENTO_GOP procedure
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function budgetCalculation($data, $nrLista) {
        try {
            $procedure = new StoredProcedure($this->connection, 'CALCULA_ORCAMENTO_GOP');
            
            $procedure->addParam(new Param('P_NRANOORCAGOP', Param::PARAM_INPUT, $data['NRANOORCAGOP'], Param::PARAM_TYPE_INT));
	        $procedure->addParam(new Param('P_IDPERIODOORCA', Param::PARAM_INPUT, $data['IDPERIODOORCA'], Param::PARAM_TYPE_STR));
	        $procedure->addParam(new Param('P_NRPERIODOORCA', Param::PARAM_INPUT, $data['NRPERIODOORCA'], Param::PARAM_TYPE_INT));
	        $procedure->addParam(new Param('P_IDTIPOORCAOP', Param::PARAM_INPUT, $data['IDTIPOORCAOP'], Param::PARAM_TYPE_STR));
	        
	        $procedure->addParam(new Param('P_PERCENTUAL', Param::PARAM_INPUT, $data['CALCULATION'] === "P" ? "S" : "N", Param::PARAM_TYPE_STR));
	        $procedure->addParam(new Param('P_LISTAFIL', Param::PARAM_INPUT, $nrLista, Param::PARAM_TYPE_STR));
	        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR)); 
	        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT,null,Param::PARAM_TYPE_STR));
	        $procedure->execute();
        } catch (Exception $ex) {
            throw $ex;
        }
    }

    /**
    * Update responsible for applying changes made to the budget details.
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function changeBudgetDetailMonth($params) {
        try {
            $update = "UPDATE ITDETALORCAMESGOP
                          SET VRORCAMENTO = :VRORCAMENTO,
                              QTORCAMENTO = :QTORCAMENTO,
                              VRPRECOBASE = :VRPRECOBASE
                        WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP
                          AND NRORG = :NRORG";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        };
    }

    /**
    * Update responsible for updating the monthly totalizers.
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function updateTotalMonth($params) {
        try {
            $update = "UPDATE ITORCAMESGOP
                          SET VRORCAMENTO = (SELECT SUM(VRORCAMENTO)
                                               FROM ITDETALORCAMESGOP
                                              WHERE NRITORCAMESGOP = :NRITORCAMESGOP)
                        WHERE NRITORCAMESGOP = :NRITORCAMESGOP
                          AND NRORG = :NRORG";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        };
    }
    
    /**
    * 
    * 
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function cleanDetailing($data) {
        try {
            $paramsUpd = [
                "NRITORCAMESGOP" => $data["NRITORCAMESGOP"]
            ];
            
            $update = "UPDATE DIETSERVORCAGOP
                          SET QTCOMDIETSEG = NULL,
                              QTCOMDIETTER = NULL,
                              QTCOMDIETQUA = NULL,
                              QTCOMDIETQUI = NULL,
                              QTCOMDIETSEX = NULL,
                              QTCOMDIETSAB = NULL,
                              QTCOMDIETDOM = NULL,
                              VRCUSTPADR = NULL
                        WHERE NRITDETALORCAMESGOP IN (SELECT NRITDETALORCAMESGOP
                                                        FROM ITDETALORCAMESGOP
                                                       WHERE NRITORCAMESGOP = :NRITORCAMESGOP)";
            
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
                
            $update = "UPDATE ITDETALORCAMESGOP
                          SET VRORCAMENTO = 0,
                              QTORCAMENTO = 0,
                              VRCUSTPADR = 0
                        WHERE NRITORCAMESGOP = :NRITORCAMESGOP";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
                
            $update = "UPDATE ITORCAMESGOP SET VRORCAMENTO = 0
                        WHERE NRITORCAMESGOP = :NRITORCAMESGOP";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
        } catch (Exception $ex) {
            throw $ex;
        }
    }

    /**
    * Update responsible for applying the change to the total value of the detail option.
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function changeBudgetDetMonthTotal($params) {
        try {
            $update = "UPDATE ITORCAMESGOP
                          SET VRORCAMENTO = :VRORCAMENTO
                        WHERE NRITORCAMESGOP = :NRITORCAMESGOP
                          AND NRORG = :NRORG";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        };
    }
  
    public function gop10000_importServiceCust($params) {
        try {
            $update = "UPDATE (SELECT I.NRITDETALORCAMESGOP, S.QTCOMESTISEG QTSEGS, S.QTCOMESTITER QTTERS,
                                      S.QTCOMESTIQUA QTQUAS, S.QTCOMESTIQUI QTQUIS,
                                      S.QTCOMESTISEX QTSEXS, S.QTCOMESTISAB QTSABS, S.QTCOMESTIDOM QTDOMS,
                                      S.QTCLIESTISEG, S.QTCLIESTITER, S.QTCLIESTIQUA, S.QTCLIESTIQUI,
                                      S.QTCLIESTISEX, S.QTCLIESTISAB, S.QTCLIESTIDOM,
                                      I.QTCOMESTISEG, I.QTCOMESTITER, I.QTCOMESTIQUA, I.QTCOMESTIQUI,
                                      I.QTCOMESTISEX, I.QTCOMESTISAB, I.QTCOMESTIDOM
                                 FROM ITDETALORCAMESGOP I INNER JOIN SERVICO S ON S.CDFILIAL = I.CDFILIAL
                                                                              AND S.CDSERVFILI = I.CDSERVFILI
                                WHERE I.NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP)
                          SET QTCOMESTISEG = QTSEGS,
                              QTCOMESTITER = QTTERS,
                              QTCOMESTIQUA = QTQUAS,
                              QTCOMESTIQUI = QTQUIS,
                              QTCOMESTISEX = QTSEXS,
                              QTCOMESTISAB = QTSABS,
                              QTCOMESTIDOM = QTDOMS";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        }
    }
  
    public function gop10000_importServiceDiners($params) {
        $diet = 
            "SELECT MAX(NRITDETALORCAMESGOP) DIET
               FROM DIETSERVORCAGOP
              WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP";
        try {
            $stmt = $this->connection->prepare($diet);
            $stmt->execute($params);
            $records = $stmt->fetchAll();
            
            if (isset($records) && !empty($records) && isset($records[0]["DIET"])) {
                $update = "UPDATE (SELECT DO.NRDIETSERVORCAGOP, DO.QTCOMDIETSEG, DO.QTCOMDIETTER,
                                          DO.QTCOMDIETQUA, DO.QTCOMDIETQUI, DO.QTCOMDIETSEX,
                                          DO.QTCOMDIETSAB, DO.QTCOMDIETDOM, D.QTCOMDIETSEG QTSEGD,
                                          D.QTCOMDIETTER QTTERD, D.QTCOMDIETQUA QTQUAD,
                                          D.QTCOMDIETQUI QTQUID, D.QTCOMDIETSEX QTSEXD,
                                          D.QTCOMDIETSAB QTSABD, D.QTCOMDIETDOM QTDOMD
                                     FROM DIETSERVORCAGOP DO INNER JOIN DIETSERV D ON D.CDFILIAL = DO.CDFILIAL
                                                                                  AND D.CDSERVFILI = DO.CDSERVFILI
                                                                                  AND D.CDDIETPAD = DO.CDDIETPAD
                                    WHERE DO.NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP)
                              SET QTCOMDIETSEG = QTSEGD,
                                  QTCOMDIETTER = QTTERD,
                                  QTCOMDIETQUA = QTQUAD,
                                  QTCOMDIETQUI = QTQUID,
                                  QTCOMDIETSEX = QTSEXD,
                                  QTCOMDIETSAB = QTSABD,
                                  QTCOMDIETDOM = QTDOMD";
                                  
                $stmt = $this->connection->prepare($update);
                $stmt->execute($params);
                
                $update = "UPDATE ITDETALORCAMESGOP I
                              SET VRORCAMENTO = (SELECT SUM(NVL(D.QTCOMDIETSEG,0) + NVL(D.QTCOMDIETTER,0)
                                                        + NVL(D.QTCOMDIETQUA,0) + NVL(D.QTCOMDIETQUI,0)
                                                        + NVL(D.QTCOMDIETSEX,0) + NVL(D.QTCOMDIETSAB,0)
                                                        + NVL(D.QTCOMDIETDOM,0)) * (SUM(D.VRCUSTPADR)/COUNT(1))
                                                   FROM DIETSERVORCAGOP D
                                                  WHERE D.NRITDETALORCAMESGOP = I.NRITDETALORCAMESGOP),
                                  QTORCAMENTO = (SELECT SUM(NVL(D.QTCOMDIETSEG,0) + NVL(D.QTCOMDIETTER,0)
                                                        + NVL(D.QTCOMDIETQUA,0) + NVL(D.QTCOMDIETQUI,0)
                                                        + NVL(D.QTCOMDIETSEX,0) + NVL(D.QTCOMDIETSAB,0)
                                                        + NVL(D.QTCOMDIETDOM,0))
                                                   FROM DIETSERVORCAGOP D
                                                  WHERE D.NRITDETALORCAMESGOP = I.NRITDETALORCAMESGOP),
                                  VRCUSTPADR = (SELECT SUM(D.VRCUSTPADR)/COUNT(1)
                                                  FROM DIETSERVORCAGOP D
                                                 WHERE D.NRITDETALORCAMESGOP = I.NRITDETALORCAMESGOP),
                                  VRPRECOBASE = (SELECT SUM(D.VRCUSTPADR)/COUNT(1)
                                                   FROM DIETSERVORCAGOP D
                                                  WHERE D.NRITDETALORCAMESGOP = I.NRITDETALORCAMESGOP)
                            WHERE I.NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP";
                                  
                $stmt = $this->connection->prepare($update);
                $stmt->execute($params);
            } else {
                $update = "UPDATE (SELECT I.NRITDETALORCAMESGOP, S.QTCOMESTISEG QTSEGS, S.QTCOMESTITER QTTERS,
                                          S.QTCOMESTIQUA QTQUAS, S.QTCOMESTIQUI QTQUIS,
                                          S.QTCOMESTISEX QTSEXS, S.QTCOMESTISAB QTSABS, S.QTCOMESTIDOM QTDOMS,
                                          S.QTCLIESTISEG, S.QTCLIESTITER, S.QTCLIESTIQUA, S.QTCLIESTIQUI,
                                          S.QTCLIESTISEX, S.QTCLIESTISAB, S.QTCLIESTIDOM,
                                          I.QTCOMESTISEG, I.QTCOMESTITER, I.QTCOMESTIQUA, I.QTCOMESTIQUI,
                                          I.QTCOMESTISEX, I.QTCOMESTISAB, I.QTCOMESTIDOM
                                     FROM ITDETALORCAMESGOP I INNER JOIN SERVICO S ON S.CDFILIAL = I.CDFILIAL
                                                                                  AND S.CDSERVFILI = I.CDSERVFILI
                                    WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP)
                              SET QTCOMESTISEG = QTSEGS,
                                  QTCOMESTITER = QTTERS,
                                  QTCOMESTIQUA = QTQUAS,
                                  QTCOMESTIQUI = QTQUIS,
                                  QTCOMESTISEX = QTSEXS,
                                  QTCOMESTISAB = QTSABS,
                                  QTCOMESTIDOM = QTDOMS";
            }
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        }
    }
    
    /**
    * 
    * 
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function gop10000_cleanService($data) {
        try {
            $paramsUpd = [
                "NRITDETALORCAMESGOP" => $data["NRITDETALORCAMESGOP"]
            ];
            
            if ($data["NRITDETALORCAMESGOP"]) {
                $update = "UPDATE DIETSERVORCAGOP
                              SET QTCOMDIETSEG = NULL,
                                  QTCOMDIETTER = NULL,
                                  QTCOMDIETQUA = NULL,
                                  QTCOMDIETQUI = NULL,
                                  QTCOMDIETSEX = NULL,
                                  QTCOMDIETSAB = NULL,
                                  QTCOMDIETDOM = NULL,
                                  VRCUSTPADR = NULL
                            WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP";
                
                $stmt = $this->connection->prepare($update);
                $stmt->execute($paramsUpd);
            }
                
            $update = "UPDATE ITDETALORCAMESGOP
                          SET VRORCAMENTO = 0,
                              QTORCAMENTO = 0,
                              VRCUSTPADR = 0,
                              VRPRECOBASE = 0
                        WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
            
            $update = "UPDATE ITORCAMESGOP M
                          SET M.VRORCAMENTO = (SELECT SUM(VRORCAMENTO)
                                                 FROM ITDETALORCAMESGOP
                                                WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP)
                        WHERE M.NRITORCAMESGOP = (SELECT NRITORCAMESGOP
                                                    FROM ITDETALORCAMESGOP
                                                   WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP)";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsUpd);
        } catch (Exception $ex) {
            throw $ex;
        }
    }/**
    * Make changes to the monthly budget
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function gop10000_saveChangeServiceDetail($params) {
        $update = "UPDATE DIETSERVORCAGOP
                      SET QTCOMDIETSEG = :QTSEG,
                          QTCOMDIETTER = :QTTER,
                          QTCOMDIETQUA = :QTQUA,
                          QTCOMDIETQUI = :QTQUI,
                          QTCOMDIETSEX = :QTSEX,
                          QTCOMDIETSAB = :QTSAB,
                          QTCOMDIETDOM = :QTDOM,
                          VRCUSTPADR = :VRCUSTPADR
                    WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP
                      AND CDDIETPAD = :CDDIETPAD";
        
        try {
            $stmt = $this->connection->prepare($update);
            $stmt->execute($params);
            
            $paramsDet = [
                "NRITDETALORCAMESGOP" => $params['NRITDETALORCAMESGOP']
            ];
            
            $update = "UPDATE ITDETALORCAMESGOP I
                          SET VRORCAMENTO = (SELECT SUM(NVL(D.QTCOMDIETSEG,0) + NVL(D.QTCOMDIETTER,0)
                                                    + NVL(D.QTCOMDIETQUA,0) + NVL(D.QTCOMDIETQUI,0)
                                                    + NVL(D.QTCOMDIETSEX,0) + NVL(D.QTCOMDIETSAB,0)
                                                    + NVL(D.QTCOMDIETDOM,0)) * (SUM(D.VRCUSTPADR)/COUNT(1))
                                               FROM DIETSERVORCAGOP D
                                              WHERE D.NRITDETALORCAMESGOP = I.NRITDETALORCAMESGOP),
                              QTORCAMENTO = (SELECT SUM(NVL(D.QTCOMDIETSEG,0) + NVL(D.QTCOMDIETTER,0)
                                                    + NVL(D.QTCOMDIETQUA,0) + NVL(D.QTCOMDIETQUI,0)
                                                    + NVL(D.QTCOMDIETSEX,0) + NVL(D.QTCOMDIETSAB,0)
                                                    + NVL(D.QTCOMDIETDOM,0))
                                               FROM DIETSERVORCAGOP D
                                              WHERE D.NRITDETALORCAMESGOP = I.NRITDETALORCAMESGOP),
                              VRCUSTPADR = (SELECT SUM(D.VRCUSTPADR)/COUNT(1)
                                              FROM DIETSERVORCAGOP D
                                             WHERE D.NRITDETALORCAMESGOP = I.NRITDETALORCAMESGOP),
                              VRPRECOBASE = (SELECT SUM(D.VRCUSTPADR)/COUNT(1)
                                               FROM DIETSERVORCAGOP D
                                              WHERE D.NRITDETALORCAMESGOP = I.NRITDETALORCAMESGOP)
                        WHERE I.NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP";
                          
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsDet);
            
            $update = "UPDATE ITORCAMESGOP M
                          SET M.VRORCAMENTO = (SELECT SUM(TD.VRORCAMENTO)
                                                 FROM ITORCAMESGOP IM, ITDETALORCAMESGOP RD, ITDETALORCAMESGOP TD
                                                WHERE IM.NRITORCAMESGOP = RD.NRITORCAMESGOP
                                                  AND IM.NRITORCAMESGOP = TD.NRITORCAMESGOP
                                                  AND RD.NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP)
                        WHERE M.NRITORCAMESGOP = (SELECT NRITORCAMESGOP
                                                    FROM ITDETALORCAMESGOP
                                                   WHERE NRITDETALORCAMESGOP = :NRITDETALORCAMESGOP)";
                
            $stmt = $this->connection->prepare($update);
            $stmt->execute($paramsDet);
        } catch (Exception $ex) {
            throw $ex;
            
        } catch (Exception $ex) {
            throw $ex;
        };
    }

    public function repeatBudget($data) {
        try {
            $params = [
                "CDFILIAL"     => $data["CDFILIAL"],
                "IDTIPOORCAOP" => $data["IDTIPOORCAOP"],
                "NRANOORCAGOP" => $data["NRANOORCAGOP"],
                "MES"          => $data["MES"],
                "FIXEDVALUE"   => $data["FIXEDVALUE"],
                "CDOPERADOR"   => $this->environment->getCdOperador(),
            ];
            
            $script = "DECLARE
                         V_REF DATE;
                         V_VRDET NUMBER;
                       BEGIN
                          V_REF := NULL;
                          FOR C_MESREF IN (
                             SELECT I.NRITEMORCAOP,
                                    RETORNA_DIA_UTIL(I.CDFILIAL,M.DTORCAMENTO,ADD_MONTHS(M.DTORCAMENTO,1)-1) DIAS,
                                    D.NRITDETALORCAMESGOP, D.VRORCAMENTO, D.VRPERCENTUAL, D.QTORCAMENTO,
                                    D.CDPRODUTO, D.CDTIPOOPER, D.CDFILIALMO, D.IDENTRSAIDOP, D.CDORCAMENTOMO,
                                    D.CDDESPADMFAT, D.CDSERVFILI, D.VRPRECOBASE, D.NRDIASUTEIS, D.CDCONTCTBL,
                                    D.CDINVEST, D.CDTPCTPAGAR, D.CDSUBGRPROD, D.CDDESPPREV, D.CDIMPOSTO,
                                    D.VRTOTALGRUP, D.CDGRUPPROD, D.CDFATUDIVER, D.CDFILICONTRIB, D.CDFILIAL,
                                    D.QTCOMESTISEG, D.QTCOMESTITER, D.QTCOMESTIQUA, D.QTCOMESTIQUI, D.QTCOMESTISEX,
                                    D.QTCOMESTISAB, D.QTCOMESTIDOM, D.VRCUSTPADR,
                                    M.VRORCAMENTO VRORCAMENTOMES, M.QTORCAMENTO QTORCAMENTOMES, M.VRTOTALMES,
                                    M.DTORCAMENTO, O.CDFILIAL CDFILIALORCA
                               FROM ITEMORCAGOP I INNER JOIN ORCAMENTOGOP O ON O.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                                           AND O.NRANOORCAGOP = :NRANOORCAGOP
                                                  INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                                                                           AND TO_NUMBER(TO_CHAR(M.DTORCAMENTO,'MM')) = TO_NUMBER(:MES)
                                                  LEFT  JOIN ITDETALORCAMESGOP D ON D.NRITORCAMESGOP = M.NRITORCAMESGOP
                              WHERE I.CDFILIAL = :CDFILIAL
                                AND O.IDVERSORCAGOP = 'NEXT'
                                AND IDTIPOORCAOP = :IDTIPOORCAOP) LOOP
                                
                             V_REF := C_MESREF.DTORCAMENTO;
                             
                             SELECT NVL(SUM(VRORCAMENTO),0)
                               INTO V_VRDET
                               FROM ITDETALORCAMESGOP
                              WHERE NRITDETALORCAMESGOP = C_MESREF.NRITDETALORCAMESGOP;
      
                             IF C_MESREF.NRITDETALORCAMESGOP IS NULL OR (V_VRDET = 0 AND C_MESREF.VRORCAMENTOMES <> 0) THEN
                                V_REF := NULL;
                                UPDATE ITORCAMESGOP
                                   SET VRORCAMENTO = CASE WHEN :FIXEDVALUE = 'Y' THEN C_MESREF.VRORCAMENTOMES
                                                          ELSE (C_MESREF.VRORCAMENTOMES/C_MESREF.DIAS) * 
                                                            RETORNA_DIA_UTIL(C_MESREF.CDFILIALORCA,DTORCAMENTO,ADD_MONTHS(DTORCAMENTO,1)-1)
                                                     END,
                                       -- QTORCAMENTO = C_MESREF.QTORCAMENTOMES,
                                       -- VRTOTALMES = C_MESREF.VRTOTALMES,
                                       CDOPERULTATU = :CDOPERADOR,
                                       DTULTATU = SYSDATE
                                 WHERE NRITEMORCAOP = C_MESREF.NRITEMORCAOP
                                   AND TO_NUMBER(TO_CHAR(DTORCAMENTO,'YYYYMM')) > TO_NUMBER(TO_CHAR(C_MESREF.DTORCAMENTO,'YYYYMM'));
                             ELSE
                                FOR C_MES IN (SELECT M.NRITORCAMESGOP, M.DTORCAMENTO DTINI, ADD_MONTHS(M.DTORCAMENTO,1)-1 DTFIN,
                                                     RETORNA_DIA_UTIL(C_MESREF.CDFILIALORCA,M.DTORCAMENTO,ADD_MONTHS(M.DTORCAMENTO,1)-1) DIAS
                                                FROM ITORCAMESGOP M
                                               WHERE NRITEMORCAOP = C_MESREF.NRITEMORCAOP
                                                 AND TO_NUMBER(TO_CHAR(M.DTORCAMENTO,'YYYYMM')) > TO_NUMBER(TO_CHAR(C_MESREF.DTORCAMENTO,'YYYYMM'))) LOOP
                                
                                   IF C_MESREF.CDSERVFILI IS NOT NULL THEN
                                      RAISE_APPLICATION_ERROR(-20501,'TESTE',TRUE);
                                   ELSE
                                      UPDATE ITDETALORCAMESGOP
                                         SET VRORCAMENTO = 0,
                                             QTORCAMENTO = 0,
                                             VRPRECOBASE = 0,
                                             CDOPERULTATU = :CDOPERADOR,
                                             DTULTATU = SYSDATE
                                       WHERE NRITORCAMESGOP = C_MES.NRITORCAMESGOP;
                                   
                                      UPDATE ITDETALORCAMESGOP
                                         SET VRORCAMENTO = CASE WHEN :FIXEDVALUE = 'Y' THEN C_MESREF.VRORCAMENTO
                                                                ELSE (C_MESREF.VRORCAMENTO/C_MESREF.DIAS) * C_MES.DIAS
                                                           END,
                                             QTORCAMENTO = C_MESREF.QTORCAMENTO,
                                             VRPRECOBASE = CASE WHEN :FIXEDVALUE = 'Y' OR C_MESREF.QTORCAMENTO = 0 THEN C_MESREF.VRPRECOBASE
                                                                ELSE (C_MESREF.VRORCAMENTO/C_MESREF.DIAS) * C_MES.DIAS
                                                           END / C_MESREF.QTORCAMENTO,
                                             -- VRTOTALGRUP = 
                                             -- VRCUSTPADR   = C_MESREF.VRCUSTPADR,
                                             CDOPERULTATU = :CDOPERADOR,
                                             DTULTATU = SYSDATE
                                         WHERE NRITORCAMESGOP = C_MES.NRITORCAMESGOP
                                           AND NVL(CDPRODUTO,'T') = NVL(C_MESREF.CDPRODUTO,'T')
                                           AND NVL(CDTIPOOPER,'T') = NVL(C_MESREF.CDTIPOOPER,'T')
                                           AND NVL(CDFILIALMO,'T') = NVL(C_MESREF.CDFILIALMO,'T')
                                           AND NVL(IDENTRSAIDOP,'T') = NVL(C_MESREF.IDENTRSAIDOP,'T')
                                           AND NVL(CDORCAMENTOMO,'T') = NVL(C_MESREF.CDORCAMENTOMO,'T')
                                           AND NVL(CDDESPADMFAT,'T') = NVL(C_MESREF.CDDESPADMFAT,'T')
                                           AND NVL(CDSERVFILI,'T') = NVL(C_MESREF.CDSERVFILI,'T')
                                           AND NVL(CDCONTCTBL,'T') = NVL(C_MESREF.CDCONTCTBL,'T')
                                           AND NVL(CDINVEST,'T') = NVL(C_MESREF.CDINVEST,'T')
                                           AND NVL(CDTPCTPAGAR,'T') = NVL(C_MESREF.CDTPCTPAGAR,'T')
                                           AND NVL(CDSUBGRPROD,'T') = NVL(C_MESREF.CDSUBGRPROD,'T')
                                           AND NVL(CDDESPPREV,'T') = NVL(C_MESREF.CDDESPPREV,'T')
                                           AND NVL(CDIMPOSTO,'T') = NVL(C_MESREF.CDIMPOSTO,'T')
                                           AND NVL(CDGRUPPROD,'T') = NVL(C_MESREF.CDGRUPPROD,'T')
                                           AND NVL(CDFATUDIVER,'T') = NVL(C_MESREF.CDFATUDIVER,'T')
                                           AND NVL(CDFILICONTRIB,'T') = NVL(C_MESREF.CDFILICONTRIB,'T')
                                           AND NVL(CDFILIAL,'T') = NVL(C_MESREF.CDFILIAL,'T');
                                   END IF;
                                END LOOP;
                             END IF;
                             
                             COMMIT;
                          END LOOP;
                          
                          IF V_REF IS NOT NULL THEN
                             FOR C_MES IN (SELECT M.NRITORCAMESGOP
                                             FROM ITEMORCAGOP I INNER JOIN ORCAMENTOGOP O ON O.NRORCAMENTOGOP = I.NRORCAMENTOGOP
                                                                                         AND O.NRANOORCAGOP = :NRANOORCAGOP
                                                                INNER JOIN ITORCAMESGOP M ON M.NRITEMORCAOP = I.NRITEMORCAOP
                                            WHERE O.CDFILIAL = :CDFILIAL
                                              AND I.IDTIPOORCAOP = :IDTIPOORCAOP
                                              AND O.IDVERSORCAGOP = 'NEXT'
                                              AND TO_NUMBER(TO_CHAR(M.DTORCAMENTO,'YYYYMM')) > TO_NUMBER(TO_CHAR(V_REF,'YYYYMM'))) LOOP
                                              
                                UPDATE ITORCAMESGOP
                                   SET VRORCAMENTO = (SELECT SUM(VRORCAMENTO)
                                                        FROM ITDETALORCAMESGOP
                                                       WHERE NRITORCAMESGOP = C_MES.NRITORCAMESGOP)
                                 WHERE NRITORCAMESGOP = C_MES.NRITORCAMESGOP;
                             END LOOP;
                          END IF;
                       END;";
            
            $stmt = $this->connection->prepare($script);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        }
    }
}