import {
  Modal,
  ModalService,
  SelectMultiple,
  Select,
  LoadingService,
  Form,
} from '@zeedhi/common';
import {
  Metadata,
  Http,
  I18n,
} from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';

export class ReplicateModalController {
  private replicateModal!: Modal;
  
  public destroyReplicateModal() {
    this.replicateModal?.destroy();
  }
  
  public async openReplicateModal() {
    this.replicateModal = await ModalService.createFromJson('/modals/replicateModal', true);
    await this.replicateModal.show();
  }
  
  public closeReplicateModal() {
    this.replicateModal.hide();
  }
  
  public getFilioper() {
    const originSelect = Metadata.getInstance<Select>('CDFILIAL');
    const destinySelect = Metadata.getInstance<SelectMultiple>('CDFILDEST');
    Http.get('/getFilioper').then((response) => {
      const data = response.data.dataset.filioper;
      originSelect.datasource.updateData(data);
      destinySelect.datasource.updateData(data);
    });
  }
  
  public confirmReplicate() {
    const importForm = Metadata.getInstance<Form>('replicateForm');
    if (importForm.validate()) {
      const utilities = new Utilities();
      const dataForm = {
        CDFILIAL: importForm.value.CDFILIAL,
        CDFILDEST: importForm.value.CDFILDEST,
        NRITEMOPERACIONAL: importForm.value.NRITEMOPERACIONAL,
        TITLE: '',
        MESSAGE: I18n.translate('gop::CONFIRM_OPERATION'),
      };
      utilities.showDialog(this.replicate, dataForm, 'YN');
    }
  }
  
  public replicate(dataForm, type) {
    const utilities = new Utilities();
    dataForm.TYPE = type;
    LoadingService.show();
    Http.post('/replicateImportParams', {
      requestType: 'Row',
      row: dataForm,
    }).then(() => {
      utilities.successMessage();
      LoadingService.hide();
      utilities.hideDialogue();
      this.closeReplicateModal();
    }).catch((e) => {
      utilities.errorMessage();
      LoadingService.hide();
      this.closeReplicateModal();
      console.warn(e);
    });
  }
}
